#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

/*
This is the Sine Wave indicator as was introduced by John Ehlers in his books.
There are two versions for this indicator that differ in the way calculating the cycles, each method is from a different book:

1) Rocket Science for Traders: Digital Signal Processing Applications by John F. Ehlers (Jul 20, 2001)
2) Cybernetic Analysis for Stocks and Futures: Cutting-Edge DSP Technology to Improve Your Trading (by John F. Ehlers (Mar 29, 2004)

PLEASE:
If you change this indicator and think it is something that everyone can benefit from, please let others know
by publishing it in the Ninja Trader support forum.

Thx.
*/

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{

    /// <summary>
    /// Enter the description of your new custom indicator here
    /// </summary>
    [Description("Enter the description of your new custom indicator here")]
    public class SineWave1 : Indicator
    {
		public enum SineWaveMethodEnum
		{
			CyberneticsAnalysis,
			RocketScience
		}
		
		public enum SmoothMethodEnum
		{
			SMA,
			WMA,
			HMA,
			JMA,
			VWMA
		}
		
        #region Variables
        // Wizard generated variables
		private double alpha = 0.07; // Default setting for Alpha
		private bool drawSupportResistance = true;
		private SineWaveMethodEnum method = SineWaveMethodEnum.CyberneticsAnalysis;
		private SmoothMethodEnum smoothMethod = SmoothMethodEnum.WMA;
		private int priceSmoothPeriod = 4;
		private int jurikJMAPhase = 0;
		
		// Cybernetics Analysis variables
		private DataSeries Cycle;
		private DataSeries InstPeriod;
		private DataSeries DeltaPhase;
		private DataSeries Value1;
		private double MedianDelta;
		private double DC;

		// Rocket Science variables
		private DataSeries		detrender;
		private DataSeries		period;
		private DataSeries 		SmoothPeriod;	
		private DataSeries	    i2;
		private DataSeries		q2;
		private DataSeries	    im;
		private DataSeries	    re;
		private bool mp = false;
	
		// Common variables
		private DataSeries DCPhase;
		private int DCPeriod;
		private DataSeries Smooth;
		private DataSeries Q1;
		private DataSeries I1;
		
		private Color supportColor = Color.Magenta;
		private Color resistanceColor = Color.Blue;
		private Color currDotColor;
		private double currDotValue;
		private string dotText   = "l"; 	
		//private Font pfont = new Font("Arial", 15, FontStyle.Bold, GraphicsUnit.Point);
		private Font pfont = new Font("Wingdings", 4, FontStyle.Bold, GraphicsUnit.Point);
		private int yPixels = 0;
		private bool drawingSupport = true;
		
//		private BoolSeries Trend;
//		private int DaysInTrend;
//		private DataSeries ITrend;
//		private DataSeries Trendline;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "LeadSine"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "Sine"));
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;
			
			Smooth = new DataSeries(this);
			Cycle = new DataSeries(this);
			Q1 = new DataSeries(this);
			I1 = new DataSeries(this);
			InstPeriod = new DataSeries(this);
			DeltaPhase = new DataSeries(this);
			Value1 = new DataSeries(this);
			DCPhase = new DataSeries(this);
			detrender = new DataSeries(this);
			period = new DataSeries(this);
			SmoothPeriod = new DataSeries(this);
			i2 = new DataSeries(this);
			q2 = new DataSeries(this);
			im = new DataSeries(this);
			re = new DataSeries(this);
			
//			ITrend = new DataSeries(this);
//			Trendline = new DataSeries(this);
//			Trend = new BoolSeries(this);			
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
		protected override void OnBarUpdate()
		{
			bool res = false;
			switch(method)
			{
				case SineWaveMethodEnum.RocketScience:
					res = RocketScience();
					break;
				default:
					res = CyberneticsAnalysis();
					break;
			}
			
			if (res && DrawSupportResistance)
			{
				// If we are drawing support and now the Leadsine is BELOW the since it meand we need to draw resistance.
				if (drawingSupport && LeadSine[0] < Sine[0])
				{
					// The LeadSince crossed below the Sine meaning we have a cycle top, we draw
					// the resistance only we get a low lower than the previous one confirming the
					// down move started.
					if (Low[0] < Low[1])
					{
						currDotValue = High[1];
						currDotColor = resistanceColor;
						drawingSupport = false;
						
						//DrawText("t"+CurrentBar.ToString(), currDotValue.ToString(), 0, currDotValue+0.5, Color.Black);
					}
				}
				else if (!drawingSupport && LeadSine[0] > Sine[0])
				{
					if (High[0] > High[1])
					{
						currDotValue = Low[1];
						currDotColor = supportColor;
						drawingSupport = true;
						
						//DrawText("t"+CurrentBar.ToString(), currDotValue.ToString(), 0, currDotValue-0.5, Color.Black);
					}				
				}
				
     			DrawText("dot"+CurrentBar.ToString(), true, dotText, 0, currDotValue, yPixels, currDotColor, pfont, StringAlignment.Center, Color.Empty, Color.Empty, 10) ;
				//DrawDot("dot"+CurrentBar.ToString(), 0, currDotValue, currDotColor);
			}
		}
		
        protected bool CyberneticsAnalysis()
        {
			if (CurrentBar < 3) return false;
			
			Smooth.Set((Median[0] + 2*Median[1] + 2*Median[2] + Median[3])/6);
			
			if (CurrentBar < 6)
			{
				Cycle.Set((Median[0] -2*Median[1] + Median[2])/4);
				return false;
			}
			else
			{
				double d;
				d = (1-0.5*alpha)*(1-0.5*alpha)*(Smooth[0]-2*Smooth[1]+Smooth[2]) + 2*(1-alpha)*Cycle[1] - (1-alpha)*(1-alpha)*Cycle[2];
				Cycle.Set(d);
			}
			
			Q1.Set((0.0962*Cycle[0] + 0.5769*Cycle[2] - 0.5769*Cycle[4] - 0.0962*Cycle[6])*(0.5 + 0.08*InstPeriod[1]));
			I1.Set(Cycle[3]);
			
			if (Q1[0] != 0 && Q1[1] != 0)
			{
				DeltaPhase.Set((I1[0]/Q1[0] - I1[1]/Q1[1]) / (1+I1[0]*I1[1]/(Q1[0]*Q1[1])));
			}
			
			if (DeltaPhase[0] < 0.1) DeltaPhase[0] = 0.1;
			if (DeltaPhase[0] > 1.1) DeltaPhase[0] = 1.1;
		
			MedianDelta = GetMedian(DeltaPhase, 5);
			
			if (MedianDelta == 0)
				DC = 15;
			else
				DC = 6.28318 / MedianDelta + 0.5;

			InstPeriod.Set(0.33*DC + 0.67*InstPeriod[1]);
			Value1[0] = 0.15*InstPeriod[0] + 0.85*Value1[1];
			
			DCPeriod = (int)Math.Floor(Value1[0]);
			
			double RealPart = 0, ImagPart = 0;
			for (int i = 0; i < Math.Min(DCPeriod, CurrentBar+1); ++i)
			{
				RealPart = RealPart + Math.Sin((Math.PI/180)*(360*i / DCPeriod)) * Cycle[i];
				ImagPart = ImagPart + Math.Cos((Math.PI/180)*(360*i / DCPeriod)) * Cycle[i];
			}
			
			if (Math.Abs(ImagPart) > 0.001) DCPhase[0] = (180/Math.PI)*Math.Atan(RealPart / ImagPart);
			if (Math.Abs(ImagPart) <= 0.001) DCPhase[0] = 90 * Math.Sign(RealPart);
			
			DCPhase[0] += 90;
			
			if (ImagPart < 0) DCPhase[0] += 180;
			if (DCPhase[0] > 315) DCPhase[0] -= 360;
			
			Sine.Set(Math.Sin((Math.PI/180)*(DCPhase[0])));
			LeadSine.Set(Math.Sin((Math.PI/180)*(DCPhase[0]+45)));
			
			return true;			
    	}

		private bool RocketScience()
		{		
			// need 7 Bars to start computing.
			if (CurrentBar < 3)
				return false;
		
			Smooth[0] = GetSmoothedPrice(Median);
		
			if (CurrentBar < 6)
				return false;
			
			detrender[0] = (0.0962 * Smooth[0] + 0.5769 * Smooth[2] - 
						   0.5769 * Smooth[4] - 0.0962 * Smooth[6]) * (0.075*period[1] + 0.54);

			// Compute InPhase and Quadrature components
			Q1.Set((0.0962 * detrender[0] + 0.5769 * detrender[2] - 
				    0.5769 * detrender[4] - 0.0962 * detrender[6]) * (0.075 * period[1] + 0.54));

			I1.Set(detrender[3]);

			// Advance the phase of i1 and q1 by 90}
			double jI = (0.0962 * I1[0] + 0.5769 * I1[2] - 0.5769 * I1[4] - 0.0962 * I1[6]) * (0.075 * period[1] + 0.54);
			double jQ = (0.0962 * Q1[0] + 0.5769 * Q1[2] - 0.5769 * Q1[4] - 0.0962 * Q1[6]) * (0.075 * period[1] + 0.54);

			// Phasor addition for 3 bar averaging}
			i2.Set(I1[0] - jQ);
			q2.Set(Q1[0] + jI);

			// Smooth the I and Q components before applying the discriminator
			i2.Set(0.2 * i2[0] + 0.8 * i2[1]);
			q2.Set(0.2 * q2[0] + 0.8 * q2[1]);

			// Homodyne Discriminator
			re.Set(i2[0] * i2[1] + q2[0] * q2[1]);
			im.Set(i2[0] * q2[1] - q2[0] * i2[1]);

			re.Set(0.2 * re[0] + 0.8 * re[1]);
			im.Set(0.2 * im[0] + 0.8 * im[1]);

			if (im[0] != 0 && re[0] != 0) period.Set(360 / ((180/Math.PI)* Math.Atan(im[0]/re[0]))); // Convert Atan return from rad to deg
			if (period[0] > 1.5  * period[1]) period.Set(1.5  * period[1]);
			if (period[0] < 0.67 * period[1]) period.Set(0.67 * period[1]);
			if (period[0] < 6)  period.Set(6);
			if (period[0] > 50) period.Set(50);

			period.Set(0.2 * period[0] + 0.8 * period[1]);
			SmoothPeriod.Set( 0.33 * period[0] + 0.67 * SmoothPeriod[1] );
			
			// Compute dominant cycle phase
			DCPeriod = (int)Math.Floor( SmoothPeriod[0] + 0.5 );
		
			double RealPart = 0;
			double ImagPart = 0;
			for(int count = 0; count < Math.Min(DCPeriod, CurrentBar+1); count++)
			{
				RealPart += Math.Cos( (Math.PI/180) * ( 360 * count / DCPeriod) ) * Smooth[count];  // Math.Sin requires input in rad
				ImagPart += Math.Sin( (Math.PI/180) * ( 360 * count / DCPeriod) ) * Smooth[count];	  // Math.Cos requires input in rad
			}
			
			if( Math.Abs(RealPart) > 0 )
			{
				DCPhase[0] = (180/Math.PI) * Math.Atan( ImagPart / RealPart );  // Convert Atan return from rad to deg
			}
			
			if( Math.Abs(RealPart) <= 0.001 )
			{
				DCPhase[0] = (90 * Math.Sign(ImagPart));
			}
			
			DCPhase[0] += 90;
			DCPhase[0] = DCPhase[0] + (360 / SmoothPeriod[0]); 
			
			if( RealPart < 0 )
			{
				DCPhase[0] += 180;
			}
			
			if( DCPhase[0] > 315 )
			{
				DCPhase[0] = DCPhase[0] - 360;
			} 
			
			Sine.Set(Math.Sin((Math.PI/180) * DCPhase[0])*100);
			LeadSine.Set(Math.Sin((Math.PI/180) * (DCPhase[0] + 45))*100);	

			#region FFU
//			// ---------------------------------
//			ITrend[0] = 0;
//			for (int i = 0; i < Math.Min(DCPeriod,CurrentBar+1); ++i)
//			{
//				ITrend[0] += Median[i];
//			}
//			if (DCPeriod > 0) ITrend[0] = ITrend[0] / DCPeriod;
//			Trendline[0] = (4*ITrend[0] + 3*ITrend[1] + 2*ITrend[2] + ITrend[3])/10;
//			if (CurrentBar < 11) Trendline[0] = Median[0];
//			
//			Trend[0] = true;
//			
//			if (CrossAbove(Sine, LeadSine, 1)|| CrossBelow(Sine, LeadSine, 1))
//			{
//				DaysInTrend = 0;
//				Trend[0] = false;
//			}
//			
//			DaysInTrend += 1;
//			
//			if (DaysInTrend < 0.5*SmoothPeriod[0])
//				Trend[0] = false;
//			
//			if (SmoothPeriod[0] != 0 && (DCPhase[0] - DCPhase[1] > 0.67*360/SmoothPeriod[0] && DCPhase[0] - DCPhase[1] < 1.5*360/SmoothPeriod[0]))
//				Trend[0] = false;
//			
//			double smoothedPrice = WMA(Median,4)[0];
//			if (Math.Abs((smoothedPrice-Trendline[0])/Trendline[0]) >= 0.15) Trend[0] = true;
//			
//			if (Trend[0])
//			{
//				if (!Trend[1])
//				{
//					if (!mp && smoothedPrice > Trendline[0])
//					{
//						DrawDot(CurrentBar.ToString(), true, 0, Low[0]-10*TickSize, Color.Green);
//						mp=true;
//					}
//					
//					if (mp && smoothedPrice < Trendline[0])
//					{
//						DrawDot(CurrentBar.ToString(), true, 0, Low[0]-10*TickSize, Color.Red);
//						mp=false;
//					}
//				}
//				
//				if (CrossAbove(smoothedPrice, Trendline, 1))
//					DrawDot(CurrentBar.ToString(), true, 0, Low[0]-10*TickSize, Color.Green);
//
//				if (CrossBelow(smoothedPrice, Trendline, 1))
//					DrawDot(CurrentBar.ToString(), true, 0, Low[0]-10*TickSize, Color.Red);
//	
//			}
//			else
//			{
//				if (CrossAbove(LeadSine, Sine, 1))
//				{
//					DrawDot(CurrentBar.ToString(), true, 0, Low[0]-10*TickSize, Color.Green);
//				}
//				if (CrossBelow(LeadSine, Sine, 1))
//				{
//					DrawDot(CurrentBar.ToString(), true, 0, Low[0]-10*TickSize, Color.Red);
//				}				
//			}	
			#endregion			
			
			return true;
		}
		
		private double GetSmoothedPrice(IDataSeries data)
		{
			switch (SmoothMethod)
			{
				case SmoothMethodEnum.SMA:
					return SMA(data, PriceSmoothPeriod)[0];
					break;
				case SmoothMethodEnum.HMA:
					return HMA(data, PriceSmoothPeriod)[0];
					break;
				case SmoothMethodEnum.JMA:
					return JurikJMA(data, JurikJMAPhase, PriceSmoothPeriod)[0];
					break;
				case SmoothMethodEnum.VWMA:
					return VWMA(data, PriceSmoothPeriod)[0];
					break;
				case SmoothMethodEnum.WMA:
				default:
					return WMA(data, PriceSmoothPeriod)[0];
					break;
			}
		}

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LeadSine
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Sine
        {
            get { return Values[1]; }
        }

        [Description("A factor when calculating a cycle. This parameter is relevant to the CyberneticsAnalysis method only.")]
        [Category("Parameters")]
		//[XmlIgnore()]
        public double Alpha
        {
            get { return alpha; }
            set { alpha = value; }
        }
		
        [Description("Determines what method to use when calculating the sine. Two different ways were presented by Ehlers in two different books, the names of the methods are according to the book name.")]
        [Category("Parameters")]
		[XmlIgnore()]
		//[Gui.Design.DisplayNameAttribute("Sine Method")]
		public NinjaTrader.Indicator.SineWave1.SineWaveMethodEnum Method
		{
			get {return method;}
			set {method = value;}
		}
		[Browsable(false)]
		public string MethodSerialize
		{
			get { return method.ToString(); }
			set { method = (SineWaveMethodEnum)Enum.Parse(typeof(SineWaveMethodEnum), value, true); }
		}

        [Description("Determines the method for smoothing the price.")]
        [Category("Rocket Science")]
		[XmlIgnore()]
		[Gui.Design.DisplayNameAttribute("Smooth Method")]
		public NinjaTrader.Indicator.SineWave1.SmoothMethodEnum SmoothMethod
		{
			get {return smoothMethod;}
			set {smoothMethod = value;}
		}
		[Browsable(false)]
		public string SmoothMethodSerialize
		{
			get { return smoothMethod.ToString(); }
			set { smoothMethod = (SmoothMethodEnum)Enum.Parse(typeof(SmoothMethodEnum), value, true); }
		}

		
        [Description("The number of bars for smoothing the price.")]
        [Category("Rocket Science")]
		[XmlIgnore()]
		public int PriceSmoothPeriod
		{
			get {return priceSmoothPeriod;}
			set {priceSmoothPeriod = value;}
		}

        [Description("If using the JurikJMA smooth method this is the 'Phase' argument for it.")]
        [Category("Rocket Science")]
		[XmlIgnore()]
		public int JurikJMAPhase
		{
			get {return jurikJMAPhase;}
			set {jurikJMAPhase = value;}
		}
		
        [Description("Determines whether to draw the support/resistance dots.")]
        [Category("Appearance")]	
		[XmlIgnore()]
		public bool DrawSupportResistance
		{
			get {return drawSupportResistance;}
			set {drawSupportResistance = value;}
		}
		
		[XmlIgnore()]
        [Description("Determines the support dots color.")]
        [Category("Appearance")]
		[Gui.Design.DisplayNameAttribute("Support Color")]
		public Color SupportColor
		{
			get {return supportColor;}
			set {supportColor = value;}
		}
		[Browsable(false)]
		public string SupportColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(supportColor); }
			set { supportColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		[XmlIgnore()]
        [Description("Determines the resistance dots color.")]
        [Category("Appearance")]
		[Gui.Design.DisplayNameAttribute("Resistance Color")]
		public Color ResistanceColor
		{
			get {return resistanceColor;}
			set {resistanceColor = value;}
		}
		[Browsable(false)]
		public string ResistanceColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(resistanceColor); }
			set { resistanceColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SineWave1[] cacheSineWave1 = null;

        private static SineWave1 checkSineWave1 = new SineWave1();

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public SineWave1 SineWave1(double alpha, NinjaTrader.Indicator.SineWave1.SineWaveMethodEnum method)
        {
            return SineWave1(Input, alpha, method);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public SineWave1 SineWave1(Data.IDataSeries input, double alpha, NinjaTrader.Indicator.SineWave1.SineWaveMethodEnum method)
        {
            if (cacheSineWave1 != null)
                for (int idx = 0; idx < cacheSineWave1.Length; idx++)
                    if (Math.Abs(cacheSineWave1[idx].Alpha - alpha) <= double.Epsilon && cacheSineWave1[idx].Method == method && cacheSineWave1[idx].EqualsInput(input))
                        return cacheSineWave1[idx];

            lock (checkSineWave1)
            {
                checkSineWave1.Alpha = alpha;
                alpha = checkSineWave1.Alpha;
                checkSineWave1.Method = method;
                method = checkSineWave1.Method;

                if (cacheSineWave1 != null)
                    for (int idx = 0; idx < cacheSineWave1.Length; idx++)
                        if (Math.Abs(cacheSineWave1[idx].Alpha - alpha) <= double.Epsilon && cacheSineWave1[idx].Method == method && cacheSineWave1[idx].EqualsInput(input))
                            return cacheSineWave1[idx];

                SineWave1 indicator = new SineWave1();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Alpha = alpha;
                indicator.Method = method;
                Indicators.Add(indicator);
                indicator.SetUp();

                SineWave1[] tmp = new SineWave1[cacheSineWave1 == null ? 1 : cacheSineWave1.Length + 1];
                if (cacheSineWave1 != null)
                    cacheSineWave1.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSineWave1 = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SineWave1 SineWave1(double alpha, NinjaTrader.Indicator.SineWave1.SineWaveMethodEnum method)
        {
            return _indicator.SineWave1(Input, alpha, method);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.SineWave1 SineWave1(Data.IDataSeries input, double alpha, NinjaTrader.Indicator.SineWave1.SineWaveMethodEnum method)
        {
            return _indicator.SineWave1(input, alpha, method);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SineWave1 SineWave1(double alpha, NinjaTrader.Indicator.SineWave1.SineWaveMethodEnum method)
        {
            return _indicator.SineWave1(Input, alpha, method);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.SineWave1 SineWave1(Data.IDataSeries input, double alpha, NinjaTrader.Indicator.SineWave1.SineWaveMethodEnum method)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SineWave1(input, alpha, method);
        }
    }
}
#endregion
